package com.taobao.weex.ui.view;

import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.net.Uri;
import android.net.http.SslError;
import android.os.Handler;
import android.os.Message;
import android.os.Build.VERSION;
import android.text.TextUtils;
import android.view.View;
import android.webkit.JavascriptInterface;
import android.webkit.JsPromptResult;
import android.webkit.SslErrorHandler;
import android.webkit.ValueCallback;
import android.webkit.WebChromeClient;
import android.webkit.WebResourceError;
import android.webkit.WebResourceRequest;
import android.webkit.WebResourceResponse;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.FrameLayout;
import android.widget.ProgressBar;
import android.widget.FrameLayout.LayoutParams;
import androidx.annotation.Nullable;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.taobao.weex.ui.view.IWebView;
import com.taobao.weex.utils.WXLogUtils;
import io.dcloud.common.DHInterface.IApp;
import io.dcloud.common.adapter.ui.webview.WebViewFactory;
import java.lang.ref.WeakReference;
import java.util.HashMap;
import java.util.Map;

public class WXWebView implements IWebView {

   private Context mContext;
   private String mOrigin;
   private WebView mWebView;
   private ProgressBar mProgressBar;
   private boolean mShowLoading = true;
   private Handler mMessageHandler;
   private static final int POST_MESSAGE = 1;
   private static final String BRIDGE_NAME = "__WEEX_WEB_VIEW_BRIDGE";
   private static final int SDK_VERSION = VERSION.SDK_INT;
   private static final boolean DOWNGRADE_JS_INTERFACE = SDK_VERSION < 17;
   private IWebView.OnErrorListener mOnErrorListener;
   private IWebView.OnPageListener mOnPageListener;
   private IWebView.OnMessageListener mOnMessageListener;


   public WXWebView(Context context, String origin) {
      this.mContext = context;
      this.mOrigin = origin;
   }

   public View getView() {
      FrameLayout root = new FrameLayout(this.mContext);
      root.setBackgroundColor(-1);
      this.mWebView = new WebView(this.mContext);
      LayoutParams wvLayoutParams = new LayoutParams(-1, -1);
      wvLayoutParams.gravity = 17;
      this.mWebView.setLayoutParams(wvLayoutParams);
      root.addView(this.mWebView);
      this.initWebView(this.mWebView);
      this.mProgressBar = new ProgressBar(this.mContext);
      this.showProgressBar(false);
      LayoutParams pLayoutParams = new LayoutParams(-2, -2);
      this.mProgressBar.setLayoutParams(pLayoutParams);
      pLayoutParams.gravity = 17;
      root.addView(this.mProgressBar);
      this.mMessageHandler = new WXWebView.MessageHandler(this, null);
      return root;
   }

   public void destroy() {
      if(this.getWebView() != null) {
         this.getWebView().setWebViewClient((WebViewClient)null);
         this.getWebView().setWebChromeClient((WebChromeClient)null);
         this.getWebView().removeAllViews();
         this.getWebView().destroy();
         this.mWebView = null;
         this.mOnMessageListener = null;
         this.mOnPageListener = null;
         this.mOnErrorListener = null;
      }

   }

   public void loadUrl(String url) {
      if(this.getWebView() != null) {
         this.getWebView().loadUrl(url);
      }
   }

   public void loadDataWithBaseURL(String source) {
      if(this.getWebView() != null) {
         this.getWebView().loadDataWithBaseURL(this.mOrigin, source, "text/html", "utf-8", (String)null);
      }
   }

   public void reload() {
      if(this.getWebView() != null) {
         this.getWebView().reload();
      }
   }

   public void goBack() {
      if(this.getWebView() != null) {
         this.getWebView().goBack();
      }
   }

   public void goForward() {
      if(this.getWebView() != null) {
         this.getWebView().goForward();
      }
   }

   public void postMessage(Object msg) {
      if(this.getWebView() != null) {
         try {
            JSONObject e = new JSONObject();
            e.put("type", "message");
            e.put("data", msg);
            e.put("origin", this.mOrigin);
            this.evaluateJS("javascript:(function () {var initData = " + e.toString() + ";try {var event = new MessageEvent(\'message\', initData);window.dispatchEvent(event);} catch (e) {}})();");
         } catch (JSONException var3) {
            throw new RuntimeException(var3);
         }
      }
   }

   public void setShowLoading(boolean shown) {
      this.mShowLoading = shown;
   }

   public void setOnErrorListener(IWebView.OnErrorListener listener) {
      this.mOnErrorListener = listener;
   }

   public void setOnPageListener(IWebView.OnPageListener listener) {
      this.mOnPageListener = listener;
   }

   public void setOnMessageListener(IWebView.OnMessageListener listener) {
      this.mOnMessageListener = listener;
   }

   public void onActivityResult(int requestCode, int resultCode, Intent data) {
   }

   private void showProgressBar(boolean shown) {
      if(this.mShowLoading) {
         this.mProgressBar.setVisibility(shown?0:8);
      }

   }

   private void showWebView(boolean shown) {
      this.mWebView.setVisibility(shown?0:4);
   }

   @Nullable
   private WebView getWebView() {
      return this.mWebView;
   }

   private void initWebView(WebView wv) {
      WebSettings settings = wv.getSettings();
      WebViewFactory.openJSEnabled(settings, (IApp)null);
      settings.setAppCacheEnabled(true);
      settings.setUseWideViewPort(true);
      settings.setDomStorageEnabled(true);
      settings.setSupportZoom(false);
      settings.setBuiltInZoomControls(false);
      settings.setAllowFileAccess(false);
      settings.setSavePassword(false);
      wv.setWebViewClient(new WebViewClient() {
         public boolean shouldOverrideUrlLoading(WebView view, String url) {
            view.loadUrl(url);
            WXLogUtils.v("tag", "onPageOverride " + url);
            return true;
         }
         public void onPageStarted(WebView view, String url, Bitmap favicon) {
            super.onPageStarted(view, url, favicon);
            WXLogUtils.v("tag", "onPageStarted " + url);
            if(WXWebView.this.mOnPageListener != null) {
               WXWebView.this.mOnPageListener.onPageStart(url);
            }

         }
         public void onPageFinished(WebView view, String url) {
            super.onPageFinished(view, url);
            WXLogUtils.v("tag", "onPageFinished " + url);
            if(WXWebView.this.mOnPageListener != null) {
               WXWebView.this.mOnPageListener.onPageFinish(url, view.canGoBack(), view.canGoForward());
            }

            if(WXWebView.this.mOnMessageListener != null) {
               WXWebView.this.evaluateJS("javascript:(window.postMessage = function(message, targetOrigin) {if (message == null || !targetOrigin) return;" + (WXWebView.DOWNGRADE_JS_INTERFACE?"prompt(\'__WEEX_WEB_VIEW_BRIDGE://postMessage?message=\' + JSON.stringify(message) + \'&targetOrigin=\' + targetOrigin)":"__WEEX_WEB_VIEW_BRIDGE.postMessage(JSON.stringify(message), targetOrigin);") + "})");
            }

         }
         public void onReceivedError(WebView view, WebResourceRequest request, WebResourceError error) {
            super.onReceivedError(view, request, error);
            if(WXWebView.this.mOnErrorListener != null) {
               WXWebView.this.mOnErrorListener.onError("error", "page error");
            }

         }
         public void onReceivedHttpError(WebView view, WebResourceRequest request, WebResourceResponse errorResponse) {
            super.onReceivedHttpError(view, request, errorResponse);
            if(WXWebView.this.mOnErrorListener != null) {
               WXWebView.this.mOnErrorListener.onError("error", "http error");
            }

         }
         public void onReceivedSslError(WebView view, SslErrorHandler handler, SslError error) {
            super.onReceivedSslError(view, handler, error);
            if(WXWebView.this.mOnErrorListener != null) {
               WXWebView.this.mOnErrorListener.onError("error", "ssl error");
            }

         }
      });
      wv.setWebChromeClient(new WebChromeClient() {
         public void onProgressChanged(WebView view, int newProgress) {
            super.onProgressChanged(view, newProgress);
            WXWebView.this.showWebView(newProgress == 100);
            WXWebView.this.showProgressBar(newProgress != 100);
            WXLogUtils.v("tag", "onPageProgressChanged " + newProgress);
         }
         public void onReceivedTitle(WebView view, String title) {
            super.onReceivedTitle(view, title);
            if(WXWebView.this.mOnPageListener != null) {
               WXWebView.this.mOnPageListener.onReceivedTitle(view.getTitle());
            }

         }
         public boolean onJsPrompt(WebView view, String url, String text, String defaultValue, JsPromptResult result) {
            Uri uri = Uri.parse(text);
            String scheme = uri.getScheme();
            if(TextUtils.equals(scheme, "__WEEX_WEB_VIEW_BRIDGE")) {
               if(TextUtils.equals(uri.getAuthority(), "postMessage")) {
                  String message = uri.getQueryParameter("message");
                  String targetOrigin = uri.getQueryParameter("targetOrigin");
                  WXWebView.this.onMessage(message, targetOrigin);
                  result.confirm("success");
               } else {
                  result.confirm("fail");
               }

               return true;
            } else {
               return super.onJsPrompt(view, url, text, defaultValue, result);
            }
         }
      });
      if(!DOWNGRADE_JS_INTERFACE) {
         wv.addJavascriptInterface(new Object() {
            @JavascriptInterface
            public void postMessage(String message, String targetOrigin) {
               WXWebView.this.onMessage(message, targetOrigin);
            }
         }, "__WEEX_WEB_VIEW_BRIDGE");
      }

   }

   private void onMessage(String message, String targetOrigin) {
      if(message != null && targetOrigin != null && this.mOnMessageListener != null) {
         try {
            HashMap e = new HashMap();
            e.put("data", JSON.parse(message));
            e.put("origin", targetOrigin);
            e.put("type", "message");
            Message threadMessage = new Message();
            threadMessage.what = 1;
            threadMessage.obj = e;
            this.mMessageHandler.sendMessage(threadMessage);
         } catch (JSONException var5) {
            throw new RuntimeException(var5);
         }
      }

   }

   private void evaluateJS(String jsStr) {
      if(SDK_VERSION < 19) {
         this.mWebView.loadUrl(jsStr);
      } else {
         this.mWebView.evaluateJavascript(jsStr, (ValueCallback)null);
      }

   }


   private static class MessageHandler extends Handler {

      private final WeakReference mWv;


      private MessageHandler(WXWebView wv) {
         this.mWv = new WeakReference(wv);
      }

      public void handleMessage(Message msg) {
         super.handleMessage(msg);
         switch(msg.what) {
         case 1:
            if(this.mWv.get() != null && ((WXWebView)this.mWv.get()).mOnMessageListener != null) {
               ((WXWebView)this.mWv.get()).mOnMessageListener.onMessage((Map)msg.obj);
            }
         default:
         }
      }

      // $FF: synthetic method
      MessageHandler(WXWebView x0, Object x1) {
         this(x0);
      }
   }
}
